#include "usart.h"
#include "dht11.h"

#define DHT11_ERROR 255

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define F_CPU 16000000

#include <util/delay.h>
#include <avr/sleep.h>
#include <stdio.h>
#include <string.h>

uint8_t alarmOn = 0;

void set_bit(uint8_t *res, int pin)
{
	(*res) |= (1 << pin);
}

void clear_bit(uint8_t *res, int pin)
{
	(*res) &= ~(1 << pin);
}
int sleep_time = 500;

void playAlarm(uint8_t *res, uint8_t pin)
{
	(*res) |= (1 << pin); // PC0 de iesire

	if (alarmOn)
	{
		set_bit(res, pin);
		_delay_ms(sleep_time);
		clear_bit(res, pin);
		_delay_ms(sleep_time);
	}
}

void ADC_init()
{
    ADMUX = (1 << REFS0) | (1 << REFS1);
    ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

unsigned int ADC_get(uint8_t channel)
{
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

unsigned int getADC(uint8_t reg) 
{
	return ADC_get(reg);
}

void print_light()
{
	unsigned int rez = getADC(PA0); 
	char val[20]; 
	double light_val = (double)(1023 - rez) * 10 / rez;
	sprintf(val, "%i lumeni\r\n", (int)light_val);
	USART0_print(val);
}

void print_temp()
{
	int8_t temp = dht11_gettemperature();
	int8_t humidity = dht11_gethumidity();

	char res[100];
	sprintf(res, "Temperature: %d\r C\nHumidity: %d%%\r\n", temp, humidity);
	USART0_print(res);
}

/*
 * get data from dht11
 */
uint8_t dht11_getdata(uint8_t select) {
	uint8_t bits[5];
	uint8_t i,j = 0;

	memset(bits, 0, sizeof(bits));

	//reset port
	DHT11_DDR |= (1<<DHT11_INPUTPIN); //output
	DHT11_PORT |= (1<<DHT11_INPUTPIN); //high
	_delay_ms(100);

	//send request
	DHT11_PORT &= ~(1<<DHT11_INPUTPIN); //low
	_delay_ms(18);
	DHT11_PORT |= (1<<DHT11_INPUTPIN); //high
	_delay_us(1);
	DHT11_DDR &= ~(1<<DHT11_INPUTPIN); //input
	_delay_us(39);

	//check start condition 1
	if((DHT11_PIN & (1<<DHT11_INPUTPIN))) {
		return DHT11_ERROR;
	}
	_delay_us(80);
	//check start condition 2
	if(!(DHT11_PIN & (1<<DHT11_INPUTPIN))) {
		return DHT11_ERROR;
	}
	_delay_us(80);

	//read the data
	for (j=0; j<5; j++) { //read 5 byte
		uint8_t result=0;
		for(i=0; i<8; i++) {//read every bit
			while(!(DHT11_PIN & (1<<DHT11_INPUTPIN))); //wait for an high input
			_delay_us(30);
			if(DHT11_PIN & (1<<DHT11_INPUTPIN)) //if input is high after 30 us, get result
				result |= (1<<(7-i));
			while(DHT11_PIN & (1<<DHT11_INPUTPIN)); //wait until input get low
		}
		bits[j] = result;
	}

	//reset port
	DHT11_DDR |= (1<<DHT11_INPUTPIN); //output
	DHT11_PORT |= (1<<DHT11_INPUTPIN); //low
	_delay_ms(100);

	//check checksum
	if (bits[0] + bits[1] + bits[2] + bits[3] == bits[4]) {
		if (select == 0) { //return temperature
			return(bits[2]);
		} else if(select == 1){ //return humidity
			return(bits[0]);
		}
	}

	return DHT11_ERROR;
}

/*
 * get temperature (0..50C)
 */
int8_t dht11_gettemperature() {
	uint8_t ret = dht11_getdata(0);
	if(ret == DHT11_ERROR)
		return -1;
	else
		return ret;
}

/*
 * get humidity (20..90%)
 */
int8_t dht11_gethumidity() {
	uint8_t ret = dht11_getdata(1);
	if(ret == DHT11_ERROR)
		return -1;
	else
		return ret;
}

int main(int argc, char const *argv[])
{
	alarmOn = 1;
	DDRC = (1 << PC0);
	clear_bit(&PORTC, PC0);
	USART0_init();
	ADC_init();
	
	uint8_t _sent = -1;
	
	while (1)
	{
		playAlarm(&PORTC, PC0);
		char c = USART0_receive();

		/* Opreste alarama */
		if (c == 'p')
		{
			alarmOn = 0;
			clear_bit(&PORTC, PC0);
		}
		/* Start alarma */		
		if (c == 's')
		{
			alarmOn = 1;
		}

		/* Afla detalii despre lumina */
		if (c == 'l')
		{
			print_light();
		}

		if (c == 't')
		{
		 	print_temp();
		}
	}

	return 0;
}